@@ -11,18 +11,17 @@ null_to_na <- function (x)
 #' whatever) to be analysed in the C++ routines.
 #'
 #' @param graph A `data.frame` containing the edges of the graph
-#' @param components If FALSE, components are not calculated (will generally
-#' result in faster processing).
-#' @return A named vector of column numbers of `edge_id`, `from`,
-#' `to`, `d`, `w`, `xfr`, `yfr`, `xto`,
-#' `yto`, and `component`, some of which may be NA.
+#' @return A list of column numbers of `edge_id`, `from`,
+#' `to`, `d`, `w`, `time`, `xfr`, `yfr`, `xto`, `yto`, and `component`, some of
+#' which may be NA.
 #'
 #' @noRd
 dodgr_graph_cols <- function (graph)
 {
     nms <- names (graph)
-    component <- which (grepl ("comp", nms)) %>% null_to_na ()
-    if (is (graph, "dodgr_streetnet") & ncol (graph) >= 11)
+    component <- grep ("comp", nms) %>% null_to_na ()
+    if (is (graph, "dodgr_streetnet") & 
+        !is (graph, "dodgr_streetnet_sc") & ncol (graph) >= 11)
     {
         # columns are always identically structured
         edge_id <- which (nms == "edge_id")
@@ -30,13 +29,18 @@ dodgr_graph_cols <- function (graph)
         to_col <- which (nms == "to_id") %>% null_to_na ()
         d_col <- which (nms == "d")
         w_col <- which (nms == "d_weighted")
+
         xfr <- which (nms == "from_lon")
+        if (length (xfr) == 0) xfr <- NA
         yfr <- which (nms == "from_lat")
+        if (length (yfr) == 0) yfr <- NA
         xto <- which (nms == "to_lon")
+        if (length (xto) == 0) xto <- NA
         yto <- which (nms == "to_lat")
+        if (length (yto) == 0) yto <- NA
     } else
     {
-        edge_id <- which (grepl ("edge_id", nms)) %>% null_to_na ()
+        edge_id <- grep ("edge_id|edge_$", nms) %>% null_to_na ()
 
         d_col <- find_d_col (graph)
         w_col <- find_w_col (graph)
@@ -51,6 +55,7 @@ dodgr_graph_cols <- function (graph)
         if (is_graph_spatial (graph))
         {
             spcols <- find_spatial_cols (graph)
+            graph <- tbl_to_df (graph)
 
             if (!(all (apply (graph [, spcols$fr_col], 2, is.numeric)) |
                   all (apply (graph [, spcols$to_tol], 2, is.numeric))))
@@ -68,20 +73,31 @@ dodgr_graph_cols <- function (graph)
         }
     }
 
-    # This is NOT a list because it's much easier to pass as vector to C++
-    ret <- c (edge_id, fr_col, to_col, d_col, w_col, xfr, yfr, xto, yto)
-    names (ret) <- c ("edge_id", "from", "to", "d", "w",
-                      "xfr", "yfr", "xto", "yto")
-    if (!is.na (component))
+    time_col <- grep ("time", nms)
+    if (length (time_col) != 1)
     {
-        ret <- c (ret, component)
-        names (ret) [length (ret)] <- "component"
+        time_col <- grep ("time$", nms)
+        if (length (time_col) != 1)
+            time_col <- NA
     }
+    timew_col <- grep ("time_w|timew|tw", nms)
+    if (length (timew_col) != 1)
+    {
+        timew_col <- grep ("time_w|timew|^tw", nms)
+        if (length (timew_col) != 1)
+            timew_col <- NA
+    }
+
+    ret <- c (edge_id, fr_col, to_col, d_col, w_col, time_col, timew_col,
+              xfr, yfr, xto, yto, component)
+    names (ret) <- c ("edge_id", "from", "to", "d", "w", "time", "time_weighted",
+                      "xfr", "yfr", "xto", "yto", "component")
     class (ret) <- c (class (ret), "graph_columns")
 
-    # Note that these are R-style 1-indexed, so need to be converted in C++ to
-    # equivalent 0-indexed forms
-    return (ret)
+    # This is passed to many C++ routines, in which case it needs to be
+    # converted to a vector (`do.call (c, gr_cols)`), and the R-style 1-indexeso
+    # need to be converted to equivalent 0-indexed forms
+    return (as.list (ret))
 }
 
 #' convert_graph
@@ -90,16 +106,37 @@ dodgr_graph_cols <- function (graph)
 #' @noRd
 convert_graph <- function (graph, gr_cols)
 {
-    indx <- which (!is.na (gr_cols [1:5]))
-    graph <- graph [, gr_cols [1:5] [indx]]
-    names (graph) <- c ("edge_id", "from", "to", "d", "w") [indx]
+    keep_cols <- c ("edge_id", "from", "to", "d", "w", "time", "time_weighted")
+    index <- do.call (c, gr_cols [keep_cols])
+    index <- index [!is.na (index)]
+    graph <- graph [, index]
+    names (graph) <- names (index)
+
     if ("edge_id" %in% names (graph))
-        if (!is.character (graph$edge_id))
-            graph$edge_id <- paste0 (graph$edge_id)
-    if (!is.character (graph$from))
-        graph$from <- paste0 (graph$from)
-    if (!is.character (graph$to))
-        graph$to <- paste0 (graph$to)
+        graph$edge_id <- convert_to_char (graph$edge_id)
+    graph$from <- convert_to_char (graph$from)
+    graph$to <- convert_to_char (graph$to)
+
+    if (!"time_weighted" %in% names (graph))
+        graph$time_weighted <- graph$time
+
+    return (graph)
+}
+
+convert_to_char <- function (x)
+{
+    if (!is.character (x)) x <- paste0 (x)
+    return (x)
+}
+
+tbl_to_df <- function (graph)
+{
+    if (methods::is (graph, "tbl"))
+    {
+        classes <- class (graph) [!grepl ("tbl", class (graph))]
+        graph <- as.data.frame (graph)
+        class (graph) <- classes
+    }
     return (graph)
 }
 
@@ -123,33 +160,35 @@ convert_graph <- function (graph, gr_cols)
 #' v <- dodgr_vertices (graph)
 dodgr_vertices <- function (graph)
 {
-    cols <- dodgr_graph_cols (graph)
-    nms <- names (cols)
+    graph <- tbl_to_df (graph)
+
+    gr_cols <- dodgr_graph_cols (graph)
+    nms <- names (gr_cols)
     # cols are (edge_id, from, to, d, w, component, xfr, yfr, xto, yto)
     # NOTE: c (x, y), where x and y are both factors gives junk, so explicit
     # conversion required here: TODO: Find a better way?
-    from_id <- graph [[cols [which (nms == "from")] ]]
-    to_id <- graph [[cols [which (nms == "to")] ]]
-    if (is.factor (from_id))
-        from_id <- paste0 (from_id)
-    if (is.factor (to_id))
-        to_id <- paste0 (to_id)
+    if (is.factor (graph [[gr_cols$from]]))
+        graph [[gr_cols$from]] <- paste0 (graph [[gr_cols$from]])
+    if (is.factor (graph [[gr_cols$to]]))
+        graph [[gr_cols$to]] <- paste0 (graph [[gr_cols$to]])
     if (is_graph_spatial (graph))
     {
-        verts <- data.frame (id = c (from_id, to_id),
-                             x = c (graph [[cols [which (nms == "xfr")] ]],
-                                    graph [[cols [which (nms == "xto")] ]]),
-                             y = c (graph [[cols [which (nms == "yfr")] ]],
-                                    graph [[cols [which (nms == "yto")] ]]),
+        verts <- data.frame (id = c (graph [[gr_cols$from]],
+                                     graph [[gr_cols$to]]),
+                             x = c (graph [[gr_cols$xfr]],
+                                    graph [[gr_cols$xto]]),
+                             y = c (graph [[gr_cols$yfr]],
+                                    graph [[gr_cols$yto]]),
                              stringsAsFactors = FALSE)
-        if ("component" %in% nms)
-            verts$component <- graph [[cols [which (nms == "component")] ]]
+        if (!is.na (gr_cols$component))
+            verts$component <- graph [[gr_cols$component]]
     } else
     {
-        verts <- data.frame (id = c (from_id, to_id),
+        verts <- data.frame (id = c (graph [[gr_cols$from]],
+                                     graph [[gr_cols$to]]),
                              stringsAsFactors = FALSE)
-        if ("component" %in% nms)
-            verts <- cbind (verts, rep (graph$component, 2))
+        if (!is.na (gr_cols$component))
+            verts$component <- graph [[gr_cols$component]]
     }
 
     indx <- which (!duplicated (verts))
@@ -174,21 +213,23 @@ dodgr_vertices <- function (graph)
 #' graph <- dodgr_components (graph)
 dodgr_components <- function (graph)
 {
+    graph <- tbl_to_df (graph)
+
     if ("component" %in% names (graph))
         message ("graph already has a component column")
     else
     {
         gr_cols <- dodgr_graph_cols (graph)
         graph2 <- convert_graph (graph, gr_cols)
-        if (is.na (gr_cols [which (names (gr_cols) == "edge_id")]))
+        if (is.na (gr_cols$edge_id))
             graph2$edge_id <- seq (nrow (graph2))
         cns <- rcpp_get_component_vector (graph2)
 
         indx <- match (graph2$edge_id, cns$edge_id)
         component <- cns$edge_component [indx]
         # Then re-number in order to decreasing component size:
-        graph$component <- match (component, order (table (component),
-                                                    decreasing = TRUE))
+        graph$component <- match (component,
+                                  order (table (component), decreasing = TRUE))
     }
 
     return (graph)
@@ -218,6 +259,9 @@ dodgr_components <- function (graph)
 #' nrow (dodgr_vertices (graph)) # 200
 dodgr_sample <- function (graph, nverts = 1000)
 {
+    classes <- class (graph)
+    graph <- tbl_to_df (graph)
+
     fr <- find_fr_id_col (graph)
     to <- find_to_id_col (graph)
     verts <- unique (c (graph [, fr], graph [, to]))
@@ -225,9 +269,10 @@ dodgr_sample <- function (graph, nverts = 1000)
     {
         gr_cols <- dodgr_graph_cols (graph)
         graph2 <- convert_graph (graph, gr_cols)
-        indx <- match (rcpp_sample_graph (graph2, nverts), graph$edge_id)
+        indx <- match (rcpp_sample_graph (graph2, nverts), graph2$edge_id)
         graph <- graph [sort (indx), ]
     }
 
+    class (graph) <- classes
     return (graph)
 }
