@@ -71,6 +71,10 @@ class FHeap: public Heap {
         long int nComps() const { return compCount; }
         void dump() const;
 
+        double getmin() {
+            return 0.0; // MP: dummy value not implemented yet
+        }
+
     private:
         FHeapNode **trees;
         FHeapNode **nodes;
