@@ -87,6 +87,10 @@ class TriHeap : public Heap {
 
         void dump() const;
 
+        double getmin() {
+            return 0.0; // MP: dummy value not implemented yet
+        }
+
     private:
         TriHeapNode **trees;
         TriHeapNode **active;
