@@ -43,54 +43,76 @@ std::shared_ptr <HeapDesc> run_sp::getHeapImpl(const std::string& heap_type)
 struct OneDist : public RcppParallel::Worker
 {
     RcppParallel::RVector <int> dp_fromi;
-    const Rcpp::IntegerVector toi;
+    const std::vector <unsigned int> toi;
     const size_t nverts;
+    const std::vector <double> vx;
+    const std::vector <double> vy;
     const std::shared_ptr <DGraph> g;
     const std::string heap_type;
+    bool is_spatial;
 
     RcppParallel::RMatrix <double> dout;
 
     // constructor
     OneDist (
             const Rcpp::IntegerVector fromi,
-            const Rcpp::IntegerVector toi_in,
+            //const Rcpp::IntegerVector toi_in,
+            const std::vector <unsigned int> toi_in,
             const size_t nverts_in,
+            const std::vector <double> vx_in,
+            const std::vector <double> vy_in,
             const std::shared_ptr <DGraph> g_in,
             const std::string & heap_type_in,
+            const bool & is_spatial_in,
             Rcpp::NumericMatrix dout_in) :
         dp_fromi (fromi), toi (toi_in), nverts (nverts_in),
-        g (g_in), heap_type (heap_type_in), dout (dout_in)
+        vx (vx_in), vy (vy_in),
+        g (g_in), heap_type (heap_type_in), is_spatial (is_spatial_in),
+        dout (dout_in)
     {
     }
 
     // Parallel function operator
     void operator() (std::size_t begin, std::size_t end)
     {
-        std::shared_ptr<Dijkstra> dijkstra =
-            std::make_shared <Dijkstra> (nverts,
+        std::shared_ptr<PF::PathFinder> pathfinder =
+            std::make_shared <PF::PathFinder> (nverts,
                     *run_sp::getHeapImpl (heap_type), g);
         std::vector <double> w (nverts);
         std::vector <double> d (nverts);
         std::vector <int> prev (nverts);
 
+        std::vector <double> heuristic (nverts, 0.0);
+
         for (std::size_t i = begin; i < end; i++)
         {
+            unsigned int from_i = static_cast <unsigned int> (dp_fromi [i]);
             // These have to be reserved within the parallel operator function!
             std::fill (w.begin (), w.end (), INFINITE_DOUBLE);
             std::fill (d.begin (), d.end (), INFINITE_DOUBLE);
 
-            if (heap_type.find ("set") == std::string::npos)
-                dijkstra->run (d, w, prev,
-                        static_cast <unsigned int> (dp_fromi [i]));
-            else
-                dijkstra->run_set (d, w, prev,
-                        static_cast <unsigned int> (dp_fromi [i]));
-            for (long int j = 0; j < toi.size (); j++)
+            if (is_spatial)
             {
-                if (w [static_cast <size_t> (toi [j])] < INFINITE_DOUBLE)
+                double dmax = 0.0;
+                for (size_t j = 0; j < nverts; j++)
                 {
-                    dout (i, static_cast <size_t> (j)) =
-                        d [static_cast <size_t> (toi [j])];
+                    double dx = vx [j] - vx [from_i],
+                        dy = vy [j] - vy [from_i];
+                    heuristic [j] = sqrt (dx * dx + dy * dy);
+                    if (heuristic [j] > dmax)
+                        dmax = heuristic [j];
+                }
+                pathfinder->AStar (d, w, prev, heuristic, from_i, toi);
+            } else if (heap_type.find ("set") == std::string::npos)
+                pathfinder->Dijkstra (d, w, prev, from_i, toi);
+            else
+                pathfinder->Dijkstra_set (d, w, prev, from_i);
+
+            for (size_t j = 0; j < toi.size (); j++)
+            {
+                if (w [toi [j]] < INFINITE_DOUBLE)
+                {
+                    dout (i, j) = d [toi [j]];
                 }
             }
         }
@@ -142,7 +164,7 @@ size_t run_sp::get_fromi (const Rcpp::DataFrame &vert_map_in,
     return static_cast <size_t> (fromi.size ());
 }
 
-// Flows from the dijkstra output are reallocated based on matching vertex
+// Flows from the pathfinder output are reallocated based on matching vertex
 // pairs to edge indices. Note, however, that contracted graphs frequently
 // have duplicate vertex pairs with different distances. The following
 // therefore uses two maps, one to hold the ultimate index from vertex
@@ -174,12 +196,16 @@ void run_sp::make_vert_to_edge_maps (const std::vector <std::string> &from,
 Rcpp::NumericMatrix rcpp_get_sp_dists_par (const Rcpp::DataFrame graph,
         const Rcpp::DataFrame vert_map_in,
         Rcpp::IntegerVector fromi,
-        Rcpp::IntegerVector toi,
-        const std::string& heap_type)
+        Rcpp::IntegerVector toi_in,
+        const std::string& heap_type,
+        const bool is_spatial)
 {
+    std::vector <unsigned int> toi =
+        Rcpp::as <std::vector <unsigned int> > ( toi_in);
+
     Rcpp::NumericVector id_vec;
-    size_t nfrom = run_sp::get_fromi_toi (vert_map_in, fromi, toi, id_vec);
-    size_t nto = static_cast <size_t> (toi.size ());
+    size_t nfrom = run_sp::get_fromi_toi (vert_map_in, fromi, toi_in, id_vec);
+    size_t nto = toi.size ();
 
     std::vector <std::string> from = graph ["from"];
     std::vector <std::string> to = graph ["to"];
@@ -193,6 +219,13 @@ Rcpp::NumericMatrix rcpp_get_sp_dists_par (const Rcpp::DataFrame graph,
     size_t nverts = run_sp::make_vert_map (vert_map_in, vert_map_id,
             vert_map_n, vert_map);
 
+    std::vector <double> vx (nverts), vy (nverts);
+    if (is_spatial)
+    {
+        vx = Rcpp::as <std::vector <double> > (vert_map_in ["x"]);
+        vy = Rcpp::as <std::vector <double> > (vert_map_in ["y"]);
+    }
+
     std::shared_ptr <DGraph> g = std::make_shared <DGraph> (nverts);
     inst_graph (g, nedges, vert_map, from, to, dist, wt);
 
@@ -202,7 +235,8 @@ Rcpp::NumericMatrix rcpp_get_sp_dists_par (const Rcpp::DataFrame graph,
             static_cast <int> (nto), na_vec.begin ());
 
     // Create parallel worker
-    OneDist one_dist (fromi, toi, nverts, g, heap_type, dout);
+    OneDist one_dist (fromi, toi, nverts, vx, vy,
+            g, heap_type, is_spatial, dout);
 
     RcppParallel::parallelFor (0, static_cast <size_t> (fromi.length ()),
             one_dist);
@@ -217,12 +251,14 @@ Rcpp::NumericMatrix rcpp_get_sp_dists_par (const Rcpp::DataFrame graph,
 Rcpp::NumericMatrix rcpp_get_sp_dists (const Rcpp::DataFrame graph,
         const Rcpp::DataFrame vert_map_in,
         Rcpp::IntegerVector fromi,
-        Rcpp::IntegerVector toi,
+        Rcpp::IntegerVector toi_in,
         const std::string& heap_type)
 {
+    std::vector <unsigned int> toi =
+        Rcpp::as <std::vector <unsigned int> > ( toi_in);
     Rcpp::NumericVector id_vec;
-    size_t nfrom = run_sp::get_fromi_toi (vert_map_in, fromi, toi, id_vec);
-    size_t nto = static_cast <size_t> (toi.size ());
+    size_t nfrom = run_sp::get_fromi_toi (vert_map_in, fromi, toi_in, id_vec);
+    size_t nto = toi.size ();
 
     std::vector <std::string> from = graph ["from"];
     std::vector <std::string> to = graph ["to"];
@@ -239,14 +275,15 @@ Rcpp::NumericMatrix rcpp_get_sp_dists (const Rcpp::DataFrame graph,
     std::shared_ptr<DGraph> g = std::make_shared<DGraph>(nverts);
     inst_graph (g, nedges, vert_map, from, to, dist, wt);
 
-    std::shared_ptr <Dijkstra> dijkstra = std::make_shared <Dijkstra> (nverts,
-            *run_sp::getHeapImpl(heap_type), g);
+    std::shared_ptr <PF::PathFinder> pathfinder =
+        std::make_shared <PF::PathFinder> (
+            nverts, *run_sp::getHeapImpl(heap_type), g);
 
     std::vector<double> w (nverts);
     std::vector<double> d (nverts);
     std::vector<int> prev (nverts);
 
-    dijkstra->init (g); // specify the graph
+    pathfinder->init (g); // specify the graph
 
     // initialise dout matrix to NA
     Rcpp::NumericVector na_vec = Rcpp::NumericVector (nfrom * nto,
@@ -254,16 +291,17 @@ Rcpp::NumericMatrix rcpp_get_sp_dists (const Rcpp::DataFrame graph,
     Rcpp::NumericMatrix dout (static_cast <int> (nfrom),
             static_cast <int> (nto), na_vec.begin ());
 
+
     for (unsigned int v = 0; v < nfrom; v++)
     {
         Rcpp::checkUserInterrupt ();
         std::fill (w.begin(), w.end(), INFINITE_DOUBLE);
         std::fill (d.begin(), d.end(), INFINITE_DOUBLE);
 
-        dijkstra->run (d, w, prev, static_cast <unsigned int> (fromi [v]));
+        pathfinder->Dijkstra (d, w, prev,
+                static_cast <unsigned int> (fromi [v]), toi);
         for (unsigned int vi = 0; vi < nto; vi++)
         {
-            //if (toi [vi] < INFINITE_INT)
             if (w [static_cast <size_t> (toi [vi])] < INFINITE_DOUBLE)
             {
                 dout (v, vi) = d [static_cast <size_t> (toi [vi])];
@@ -296,12 +334,14 @@ Rcpp::NumericMatrix rcpp_get_sp_dists (const Rcpp::DataFrame graph,
 Rcpp::List rcpp_get_paths (const Rcpp::DataFrame graph,
         const Rcpp::DataFrame vert_map_in,
         Rcpp::IntegerVector fromi,
-        Rcpp::IntegerVector toi,
+        Rcpp::IntegerVector toi_in,
         const std::string& heap_type)
 {
+    std::vector <unsigned int> toi =
+        Rcpp::as <std::vector <unsigned int> > ( toi_in);
     Rcpp::NumericVector id_vec;
-    size_t nfrom = run_sp::get_fromi_toi (vert_map_in, fromi, toi, id_vec);
-    size_t nto = static_cast <size_t> (toi.size ());
+    size_t nfrom = run_sp::get_fromi_toi (vert_map_in, fromi, toi_in, id_vec);
+    size_t nto = toi.size ();
 
     std::vector <std::string> from = graph ["from"];
     std::vector <std::string> to = graph ["to"];
@@ -318,7 +358,8 @@ Rcpp::List rcpp_get_paths (const Rcpp::DataFrame graph,
     std::shared_ptr<DGraph> g = std::make_shared<DGraph>(nverts);
     inst_graph (g, nedges, vert_map, from, to, dist, wt);
 
-    std::shared_ptr<Dijkstra> dijkstra = std::make_shared <Dijkstra> (nverts,
+    std::shared_ptr<PF::PathFinder> pathfinder =
+        std::make_shared <PF::PathFinder> (nverts,
             *run_sp::getHeapImpl(heap_type), g);
     
     Rcpp::List res (nfrom);
@@ -326,7 +367,7 @@ Rcpp::List rcpp_get_paths (const Rcpp::DataFrame graph,
     std::vector<double> d (nverts);
     std::vector<int> prev (nverts);
 
-    dijkstra->init (g); // specify the graph
+    pathfinder->init (g); // specify the graph
 
     for (unsigned int v = 0; v < nfrom; v++)
     {
@@ -334,21 +375,22 @@ Rcpp::List rcpp_get_paths (const Rcpp::DataFrame graph,
         std::fill (w.begin(), w.end(), INFINITE_DOUBLE);
         std::fill (d.begin(), d.end(), INFINITE_DOUBLE);
 
-        dijkstra->run (d, w, prev, static_cast <unsigned int> (fromi [v]));
+        pathfinder->Dijkstra (d, w, prev,
+                static_cast <unsigned int> (fromi [v]), toi);
 
         Rcpp::List res1 (nto);
         for (unsigned int vi = 0; vi < nto; vi++)
         {
-            std::vector <unsigned int> onePath;
-            if (w [static_cast <size_t> (toi [vi])] < INFINITE_DOUBLE)
+            std::vector <int> onePath;
+            if (w [toi [vi]] < INFINITE_DOUBLE)
             {
-                int target = toi [vi]; // target can be -1!
+                int target = toi_in [vi]; // target can be -1!
                 while (target < INFINITE_INT)
                 {
                     // Note that targets are all C++ 0-indexed and are converted
                     // directly here to R-style 1-indexes.
-                    onePath.push_back (static_cast <unsigned int> (target + 1));
-                    target = prev [static_cast <size_t> (target)];
+                    onePath.push_back (target + 1);
+                    target = static_cast <int> (prev [static_cast <unsigned int> (target)]);
                     if (target < 0 || target == fromi [v])
                         break;
                 }
@@ -368,7 +410,7 @@ Rcpp::List rcpp_get_paths (const Rcpp::DataFrame graph,
 struct OneFlow : public RcppParallel::Worker
 {
     RcppParallel::RVector <int> dp_fromi;
-    const Rcpp::IntegerVector toi;
+    const std::vector <unsigned int> toi;
     const Rcpp::NumericMatrix flows;
     const std::vector <std::string> vert_name;
     const std::unordered_map <std::string, unsigned int> verts_to_edge_map;
@@ -382,7 +424,7 @@ struct OneFlow : public RcppParallel::Worker
     // constructor
     OneFlow (
             const Rcpp::IntegerVector fromi,
-            const Rcpp::IntegerVector toi_in,
+            const std::vector <unsigned int> toi_in,
             const Rcpp::NumericMatrix flows_in,
             const std::vector <std::string>  vert_name_in,
             const std::unordered_map <std::string, unsigned int> verts_to_edge_map_in,
@@ -417,8 +459,8 @@ struct OneFlow : public RcppParallel::Worker
     // Parallel function operator
     void operator() (size_t begin, size_t end)
     {
-        std::shared_ptr<Dijkstra> dijkstra =
-            std::make_shared <Dijkstra> (nverts,
+        std::shared_ptr<PF::PathFinder> pathfinder =
+            std::make_shared <PF::PathFinder> (nverts,
                     *run_sp::getHeapImpl (heap_type), g);
         std::vector <double> w (nverts);
         std::vector <double> d (nverts);
@@ -432,18 +474,17 @@ struct OneFlow : public RcppParallel::Worker
             std::fill (w.begin (), w.end (), INFINITE_DOUBLE);
             std::fill (d.begin (), d.end (), INFINITE_DOUBLE);
 
-            dijkstra->run (d, w, prev,
-                    static_cast <unsigned int> (dp_fromi [i]));
-            for (size_t j = 0; j < static_cast <size_t> (toi.size ()); j++)
+            unsigned int from_i = static_cast <unsigned int> (dp_fromi [i]);
+
+            pathfinder->Dijkstra (d, w, prev, from_i, toi);
+            for (size_t j = 0; j < toi.size (); j++)
             {
-                long int ltj = static_cast <long int> (j);
-                if (dp_fromi [i] != toi [ltj]) // Exclude self-flows
+                if (from_i != toi [j]) // Exclude self-flows
                 {
                     double flow_ij = flows (i, j);
-                    if (w [static_cast <size_t> (toi [ltj])] < INFINITE_DOUBLE &&
-                            flow_ij > 0.0)
+                    if (w [toi [j]] < INFINITE_DOUBLE && flow_ij > 0.0)
                     {
-                        int target = toi [ltj]; // can equal -1
+                        int target = static_cast <int> (toi [j]); // can equal -1
                         while (target < INFINITE_INT)
                         {
                             size_t stt = static_cast <size_t> (target);
@@ -457,7 +498,7 @@ struct OneFlow : public RcppParallel::Worker
                                 flowvec [verts_to_edge_map.at (v2)] += flow_ij;
                             }
 
-                            target = prev [stt];
+                            target = static_cast <int> (prev [stt]);
                             // Only allocate that flow from origin vertex v to all
                             // previous vertices up until the target vi
                             if (target < 0 || target == dp_fromi [i])
@@ -537,13 +578,15 @@ Rcpp::NumericVector rcpp_aggregate_files (const Rcpp::CharacterVector file_names
 void rcpp_flows_aggregate_par (const Rcpp::DataFrame graph,
         const Rcpp::DataFrame vert_map_in,
         Rcpp::IntegerVector fromi,
-        Rcpp::IntegerVector toi,
+        Rcpp::IntegerVector toi_in,
         const Rcpp::NumericMatrix flows,
         const std::string dirtxt,
         const std::string heap_type)
 {
+    std::vector <unsigned int> toi =
+        Rcpp::as <std::vector <unsigned int> > ( toi_in);
     Rcpp::NumericVector id_vec;
-    const size_t nfrom = run_sp::get_fromi_toi (vert_map_in, fromi, toi, id_vec);
+    const size_t nfrom = run_sp::get_fromi_toi (vert_map_in, fromi, toi_in, id_vec);
 
     const std::vector <std::string> from = graph ["from"];
     const std::vector <std::string> to = graph ["to"];
@@ -624,7 +667,8 @@ Rcpp::NumericVector rcpp_flows_disperse (const Rcpp::DataFrame graph,
     std::shared_ptr<DGraph> g = std::make_shared<DGraph> (nverts);
     inst_graph (g, nedges, vert_map_i, from, to, dist, wt);
 
-    std::shared_ptr <Dijkstra> dijkstra = std::make_shared <Dijkstra> (nverts,
+    std::shared_ptr <PF::PathFinder> pathfinder =
+        std::make_shared <PF::PathFinder> (nverts,
             *run_sp::getHeapImpl(heap_type), g);
 
     Rcpp::List res (nfrom);
@@ -632,6 +676,10 @@ Rcpp::NumericVector rcpp_flows_disperse (const Rcpp::DataFrame graph,
     std::vector<double> d(nverts);
     std::vector<int> prev(nverts);
 
+    std::vector <unsigned int> toi (nverts);
+    for (unsigned int i = 0; i < nverts; i++)
+        toi [i] = i;
+
     Rcpp::NumericVector aggregate_flows (from.size ()); // 0-filled by default
     for (unsigned int v = 0; v < nfrom; v++)
     {
@@ -639,7 +687,8 @@ Rcpp::NumericVector rcpp_flows_disperse (const Rcpp::DataFrame graph,
         std::fill (w.begin(), w.end(), INFINITE_DOUBLE);
         std::fill (d.begin(), d.end(), INFINITE_DOUBLE);
 
-        dijkstra->run (d, w, prev, static_cast <unsigned int> (fromi [v]));
+        pathfinder->Dijkstra (d, w, prev,
+                static_cast <unsigned int> (fromi [v]), toi);
 
         for (unsigned int vi = 0; vi < nverts; vi++)
         {
