@@ -17,6 +17,8 @@ test_that("components", {
     graph$component <- NULL
     graph <- dodgr_components (graph)
     expect_identical (comp, graph$component)
+    comp <- graph$component
+    expect_identical (comp, graph$component)
 })
 
 test_that("contract graph", {
@@ -32,15 +34,11 @@ test_that("compare heaps", {
     # Test that all dodgr calculations are faster than igraph:
     igr <- which (grepl ("igraph", ch$test))
     #expect_true (ch$elapsed [igr] == max (ch$elapsed))
-    # This actually fails on some machines (R oldrel on Windows) so:
+    # This actually fails on some machines (R oldrel on Windows) because elapsed
+    # times are sometimes all very small *and equal*, so is turned off:
     if (test_all)
     {
         #expect_true (ch$elapsed [igr] > min (ch$elapsed))
-        # TODO: igraph is now faster because of dodgr_to_igraph (#39), which now
-        # constructs a full igraph object with spatial coordinates. This in turn
-        # enables igraph to implement an A* algorithm which is naturally faster
-        # than the standard Dijkstra. TODO: #80
-        #expect_true (ch$elapsed [igr] == min (ch$elapsed))
     }
 })
 
@@ -56,18 +54,15 @@ test_that("different geometry columns", {
     gcol <- grep ("geometry", names (h2))
     names (h2) [gcol] <- "g"
     attr (h2, "sf_column") <- "g" # not necessary here but should always be done
-    expect_message (net <- weight_streetnet (h2),
-                    "The following highway types are present")
+    expect_silent (net <- weight_streetnet (h2))
     h2 <- hampi
     names (h2) [gcol] <- "geoms"
     attr (h2, "sf_column") <- "geoms"
-    expect_message (net <- weight_streetnet (h2),
-                    "The following highway types are present")
+    expect_silent (net <- weight_streetnet (h2))
     h2 <- hampi
     names (h2) [gcol] <- "geometry"
     attr (h2, "sf_column") <- "geometry"
-    expect_message (net <- weight_streetnet (h2),
-                    "The following highway types are present")
+    expect_silent (net <- weight_streetnet (h2))
     h2 <- hampi
     names (h2) [gcol] <- "xxx"
     expect_error (net <- weight_streetnet (h2),
@@ -87,10 +82,12 @@ test_that("no geom rownames", {
     h2$geometry <- g0
     hw1 <- weight_streetnet (h2)
     expect_true (!identical (h2, hw1))
-    expect_equal (ncol (hw0), 13) # has way_id column
-    expect_equal (ncol (hw1), 12) # no way_id
-    indx0 <- which (!names (hw0) %in% c ("from_id", "to_id", "way_id", "component"))
-    indx1 <- which (!names (hw1) %in% c ("from_id", "to_id", "component"))
+    expect_equal (ncol (hw0), 15) # has way_id column
+    expect_equal (ncol (hw1), 14) # no way_id
+    indx0 <- which (!names (hw0) %in% c ("from_id", "to_id", "way_id",
+                                         "component", "time", "time_weighted"))
+    indx1 <- which (!names (hw1) %in% c ("from_id", "to_id", "way_id",
+                                         "component", "time", "time_weighted"))
     expect_identical (hw0 [, indx0], hw1 [, indx1])
     # components are not identical because ones of equal size are assigned
     # random numbers, but all other columns remain identical:
@@ -99,31 +96,30 @@ test_that("no geom rownames", {
 
 test_that("keep cols", {
     hw0 <- weight_streetnet (hampi)
-    expect_equal (ncol (hw0), 13)
-    hw1 <- weight_streetnet (hampi, keep_cols = "surface")
-    expect_equal (ncol (hw1), 14)
-    expect_true ("surface" %in% names (hw1))
-    expect_false ("surface" %in% names (hw0))
+    expect_equal (ncol (hw0), 15)
+    hw1 <- weight_streetnet (hampi, keep_cols = "foot")
+    expect_equal (ncol (hw1), 16)
+    expect_true ("foot" %in% names (hw1))
+    expect_false ("foot" %in% names (hw0))
 })
 
 test_that ("weight_profiles", {
     graph0 <- weight_streetnet (hampi, wt_profile = "foot")
     graph1 <- weight_streetnet (hampi, wt_profile = 1)
-    # hampi has some undefined highway types which should be removed from graph,
-    # giving less rows in streetnet
-    expect_true (nrow (graph0) < nrow (graph1))
+    expect_equal (nrow (graph0), nrow (graph1))
     #expect_identical (graph0$d, graph1$d)
     #expect_true (!identical (graph0$d_weighted, graph1$d_weighted))
-    wtp <- dodgr::weighting_profiles [dodgr::weighting_profiles == "foot", ]
-    graph3 <- weight_streetnet (hampi, wt_profile = wtp)
+    wp <- dodgr::weighting_profiles$weighting_profiles
+    wpf <- wp [wp$name == "foot", ]
+    graph3 <- weight_streetnet (hampi, wt_profile = wpf)
     expect_identical (graph0$d_weighted, graph3$d_weighted)
 
-    wtp$value [wtp$way == "path"] <- 0.9
-    graph4 <- weight_streetnet (hampi, wt_profile = wtp)
+    wpf$value [wpf$way == "path"] <- 0.9
+    graph4 <- weight_streetnet (hampi, wt_profile = wpf)
     expect_true (!identical (graph0$d_weighted, graph4$d_weighted))
 
-    names (wtp) [3] <- "Value"
-    expect_error (graph4 <- weight_streetnet (hampi, wt_profile = wtp),
+    names (wpf) [3] <- "Value"
+    expect_error (graph4 <- weight_streetnet (hampi, wt_profile = wpf),
                   "Weighting profiles must have")
 
     g0 <- weight_streetnet (hampi)
@@ -171,6 +167,61 @@ test_that ("points to graph", {
     v <- dodgr_vertices (net)
     expect_silent (index2 <- match_pts_to_graph (v, pts))
     expect_identical (index1, index2)
+
+    colnames (pts) <- NULL
+    expect_message (index3 <- match_pts_to_graph (v, pts),
+                    "xy has no named columns; assuming order is x then y")
+    expect_identical (index1, index3)
+
+    pts <- data.frame (x = x, y = y, x2 = x)
+    expect_error (index4 <- match_pts_to_graph (v, list (pts)),
+                  "xy must be a matrix or data.frame")
+    expect_error (index4 <- match_pts_to_graph (v, pts),
+                  "xy must have only two columns")
+
+    pts <- data.frame (x = x, y = y)
+    expect_silent (index4 <- match_pts_to_graph (v, pts, connected = TRUE))
+    expect_true (!identical (index1, index4))
+
+    class (pts) <- c (class (pts), "tbl")
+    expect_silent (index5 <- match_pts_to_graph (v, pts, connected = TRUE))
+    expect_identical (index4, index5)
+
+    pts <- sf::st_as_sf (pts, coords = c (1, 2), crs = 4326)
+    expect_silent (index6 <- match_pts_to_graph (v, pts, connected = TRUE))
+    expect_identical (index4, index6)
+    expect_silent (index7 <- match_points_to_graph (v, pts, connected = TRUE))
+    expect_identical (index4, index7)
+})
+
+test_that ("graph columns", {
+    graph <- data.frame (weight_streetnet (hampi)) # rm dodgr_streetnet class
+    nf <- 100
+    nt <- 50
+    from <- sample (graph$from_id, size = nf)
+    to <- sample (graph$from_id, size = nt)
+    d <- dodgr_dists (graph, from = from, to = to)
+    graph$from_id2 <- graph$from_id
+    expect_error (d <- dodgr_dists (graph, from = from, to = to),
+                  "Unable to determine column with ID of from vertices")
+    graph$from_id2 <- NULL
+    graph$to_id2 <- graph$to_id
+    expect_error (d <- dodgr_dists (graph, from = from, to = to),
+                  "Unable to determine column with ID of to vertices")
+
+    graph <- data.frame (weight_streetnet (hampi)) # rm dodgr_streetnet class
+    graph$from_lat <- NULL
+    expect_error (d <- dodgr_dists (graph, from = from, to = to),
+                  "Unable to determine coordinate columns of graph")
+
+    graph <- data.frame (weight_streetnet (hampi)) # rm dodgr_streetnet class
+    graph$d2 <- graph$d
+    expect_error (d <- dodgr_dists (graph, from = from, to = to),
+                  "Unable to determine distance column in graph")
+    graph$d2 <- NULL
+    graph$d_wt <- graph$d_weighted
+    expect_error (d <- dodgr_dists (graph, from = from, to = to),
+                  "Unable to determine weight column in graph")
 })
 
 test_that ("get_id_cols", {
